/* Bit3_617.cc */
/* Edited by Sanshiro E. on 11 December 1999. */
/* Last updated by Sanshiro E. on 27 December 1999. */


#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include "vmedrv.h"
#include "Bit3_617.hh"


static const char *VmedrvDeviceFileName[] = {
    /* this must be consistent with TVmeTarnsferMode */
    /* (declared in ParaVmeAccess.hh) */
    "/dev/vmedrv16d16", "/dev/vmedrv16d32",
    "/dev/vmedrv24d16", "/dev/vmedrv24d32", 
    "/dev/vmedrv32d16", "/dev/vmedrv32d32"
};

static const char *VmedrvDmaDeviceFileName[] = {
    /* this must be consistent with TVmeTarnsferMode */
    /* (declared in ParaVmeAccess.hh) */
    "/dev/vmedrv16d16", "/dev/vmedrv16d32",
    "/dev/vmedrv24d16dma", "/dev/vmedrv24d32dma", 
    "/dev/vmedrv32d16dma", "/dev/vmedrv32d32dma"
};


TVmeController_Bit3_617::TVmeController_Bit3_617(void)
{
}

TVmeController_Bit3_617::~TVmeController_Bit3_617()
{
}

const char* TVmeController_Bit3_617::DeviceFileName(TVmeTransferMode TransferMode)
{
    return VmedrvDeviceFileName[TransferMode];
}

const char* TVmeController_Bit3_617::DmaDeviceFileName(TVmeTransferMode TransferMode)
{
    return VmedrvDmaDeviceFileName[TransferMode];
}

void TVmeController_Bit3_617::EnableInterruptNotification(TVmeAccessProperty* Property, int SignalId) throw(THardwareException)
{
    struct vmedrv_interrupt_property_t InterruptProperty;
    InterruptProperty.irq = Property->InterruptNumber;
    InterruptProperty.vector = Property->InterruptVector;
    InterruptProperty.signal_id = SignalId;

    int Status = ioctl(
	Property->DeviceDescriptor, 
	VMEDRV_IOC_REGISTER_INTERRUPT, 
	&InterruptProperty
    );
    if (Status < 0) {
	throw THardwareException(
            "TVmeController_Bit3_617::EnableInterruptNotification(): "
	    "ioctl(2)",
	    strerror(errno)
        );
    }

    Status = ioctl(
	Property->DeviceDescriptor, 
	VMEDRV_IOC_ENABLE_INTERRUPT
    );
    if (Status < 0) {
	throw THardwareException(
            "TVmeController_Bit3_617::EnableInterruptNotification(): "
	    "ioctl(2)",
	    strerror(errno)
        );
    }
}

void TVmeController_Bit3_617::DisableInterruptNotification(TVmeAccessProperty* Property, int SignalId) throw(THardwareException)
{
    struct vmedrv_interrupt_property_t InterruptProperty;
    InterruptProperty.irq = Property->InterruptNumber;
    InterruptProperty.vector = Property->InterruptVector;
    InterruptProperty.signal_id = SignalId;

    int Status = ioctl(
	Property->DeviceDescriptor, 
	VMEDRV_IOC_UNREGISTER_INTERRUPT, 
	&InterruptProperty
    );
    if (Status < 0) {
	throw THardwareException(
            "TVmeController_Bit3_617::DisableInterruptNotification(): "
	    "ioctl(2)",
	    strerror(errno)
        );
    }
}

