/* Kek_Camlib.hh */
/* Edited by Sanshiro E. on 6 October 1997. */
/* Last updated by Sanshiro E. on 27 December 1999. */


#include <camlib.h>
#include "Kek_Camlib.hh"


/* camlib definitions <camlib.h> */
extern "C" { 
    int CAMOPN(void);
    int CAMCLS(void);
    int CSETBR(int /* (?o?) */);
    int CSETCR(int crate);
    int CGENZ(void);
    int CGENC(void);
    int CREMI(void);
    int CSETI(void);
    int CAMACW(int naf, short *dat, int *q, int *x);
    int CAMAC(int naf, int *dat, int *q, int *x);
    int CELAM(int mask);
    int CDLAM(void);
    int CWLAM(int timeout);
    int CDMAW(int mode, int naf, unsigned short *buf, int len, int *retlen, int *err);
    int CDMAL(int mode, int naf, int *buf, int len, int *retlen, int *err);
    int CLEWAI(unsigned short *list, int lenlis, unsigned short *buf, int lenbuf, struct cc_iosb *iosb);
    int CEWAI(unsigned short *buf, int lenbuf, struct cc_iosb *iosb);
    int CLLOA(unsigned short *list, int lenlis, int *err);
    int CRLIST(char *fname, unsigned short *list, int lenlis, int *retlen, int *err);
}


TCamacController_Kek_Camlib::TCamacController_Kek_Camlib(void) throw(THardwareException)
{
    int Status = 0;

    Status += CAM_Open();
    Status += CSETBR(1);
    Status += CSETCR(0);

    Status += CGENC();
    Status += CGENZ();
    Status += CREMI();

    if (Status != 0) {
	throw THardwareException(
            "TCamacController_Kek_Camlib::TCamacController_Kek_Camlib()",
            "Initialization fault"
        );
    }

    _IsInterruptNotificationEnabled = 0;
}

TCamacController_Kek_Camlib::~TCamacController_Kek_Camlib()
{
    if (_IsInterruptNotificationEnabled) {
	try {
	    DisableInterruptNotification();
	}
	catch (THardwareException &e) {
	}
    }
    CAM_Close();
}

int TCamacController_Kek_Camlib::EnableInterruptNotification(void) throw(THardwareException)
{
    if (_IsInterruptNotificationEnabled) {
	return 0;
    }

    unsigned LamMask = ~0;
    int Status = CAM_EnableLAM(LamMask);
    if (Status != 0) {
	throw THardwareException(
            "TCamacController_Kek_Camlib::EnableLam()",
            "LAM enable fault"
        );
    }

    _IsInterruptNotificationEnabled = 1;

    return Status;
}

int TCamacController_Kek_Camlib::DisableInterruptNotification(void) throw(THardwareException)
{
    if (! _IsInterruptNotificationEnabled) {
	return 0;
    }

    int Status = CAM_DisableLAM();

    if (Status != 0) {
	throw THardwareException(
            "TCamacController_Kek_Camlib::DisableLam()",
            "LAM disable fault"
        );
    }

    return Status;
}
int TCamacController_Kek_Camlib::ReadLam(void) throw(THardwareException)
{
    throw THardwareException(
        "TCamacController_Kek_Camlib::ReadLam()",
        "This function has not been implemented yet."
    );
}

int TCamacController_Kek_Camlib::WaitLam(int TimeOut) throw(THardwareException)
{
    if (! _IsInterruptNotificationEnabled) {
	EnableInterruptNotification();
    }

    return CAM_WaitLAM(TimeOut);
}

int TCamacController_Kek_Camlib::CamacAction(int StationNumber, int Address, int Function, int &Data, int &Q, int &X) throw(THardwareException)
{
    static int Result;
    Result = CAMAC(NAF(StationNumber, Address, Function), &Data, &Q, &X);
    Data &= 0x00FFFFFF;

    return Result;
}

int TCamacController_Kek_Camlib::Initialize(void) throw(THardwareException)
{
   return CGENZ(); 
}

int TCamacController_Kek_Camlib::Clear(void) throw(THardwareException)
{
    return CGENC();
}

int TCamacController_Kek_Camlib::Inhibit(void) throw(THardwareException)
{
    return CSETI();
}

int TCamacController_Kek_Camlib::ReleaseInhibit(void) throw(THardwareException)
{
    return CREMI();
}

