/* DeviceFactory.cc */
/* Edited by Sanshiro E. on 7 May 1999. */
/* Last updated by Sanshiro E. on 9 October 1999. */


#include <string>
#include <iostream.h>
#include "RoomControllers.hh"
#include "RoomDeviceFactory.hh"
#include "RoomSoftwareDevice.hh"
#include "VmeModules.hh"
#include "CamacModules.hh"


TDeviceFactory* TDeviceFactory::_Instance = 0;

TDeviceFactory::TDeviceFactory()
{
}

TDeviceFactory::~TDeviceFactory()
{
}

TDeviceFactory* TDeviceFactory::GetInstance(void)
{
    if (_Instance == 0) {
        _Instance = new TDeviceFactory();
    }

    return _Instance;
}

string TDeviceFactory::ReducedNameOf(string Name) const
{
    // replace '-' or ' ' with '_'
    for (unsigned i = 0; i < Name.size(); i++) {
	if ((Name[i] == '-') || (Name[i] == ' ')) {
	    Name[i] = '_';
	}
    }

    // if Name contains more than three words, remove the first word.
    if (Name.find('_') != Name.rfind('_')) {
	return Name.substr(Name.find('_') + 1);
    }
    else {
	return Name;
    }
}


TVmeModule* TDeviceFactory::CreateVmeModule(const string& Name) throw(THardwareException)
{
    TVmeModule* Module = 0;
    string ReducedName = ReducedNameOf(Name);

    if (ReducedName == "Rinei_RPV130") {
        Module = new TVmeIORegister_Rinei_RPV130();
    }
    else if (ReducedName == "Hoshin_V004") {
        Module = new TVmeScaler_Hoshin_V004();
    }
    else {
        throw THardwareException(
            "TDeviceFactory::CreateVmeModule()",
            "No such vme module: " + Name
        );
    }

    return Module;
}

TCamacModule* TDeviceFactory::CreateCamacModule(const string& Name) throw(THardwareException)
{
    TCamacModule* Module = 0;
    string ReducedName = ReducedNameOf(Name);

    if (ReducedName == "Rinei_RPC022") {
        Module = new TCamacQADC_Rinei_RPC022();
    }
    else if (ReducedName == "Hoshin_C009") {
        Module = new TCamacQADC_Hoshin_C009();
    }
    else if (ReducedName == "Rinei_RPC060") {
        Module = new TCamacTDC_Rinei_RPC060();
    }
    else if (ReducedName == "Hoshin_C006") {
        Module = new TCamacTDC_Hoshin_C006();
    }
    else if (ReducedName == "Rinei_RPC081") {
        Module = new TCamacFADC_Rinei_RPC081();
    }

    else {
        throw THardwareException(
            "TDeviceFactory::CreateCamacModule()",
            "No such camac module: " + Name
        );
    }

    return Module;
}

TSoftwareModule* TDeviceFactory::CreateSoftwareModule(const string& Name) throw(THardwareException)
{
    TSoftwareModule* Module = 0;

    if (Name == "PoissonGenerator") {
        Module = new TPoissonGenerator();
    }
    else if (Name == "SoftwareCounter") {
        Module = new TSoftwareCounter();
    }
    else if (Name == "TimerInterrupter") {
        Module = new TTimerInterrupter();
    }

    else {
        throw THardwareException(
            "TDeviceFactory::CreateSoftwareModule()",
            "No such software module: " + Name
        );
    }

    return Module;
}

TVmeCrate* TDeviceFactory::CreateVmeCrate(const string& Name) throw(THardwareException)
{
    return new TVmeCrate();
}

TCamacCrate* TDeviceFactory::CreateCamacCrate(const string& Name) throw(THardwareException)
{
    return new TCamacCrate();
}

TVmeController* TDeviceFactory::CreateVmeController(const string& Name) throw(THardwareException)
{
    TVmeController* Controller = 0;
    string ReducedName = ReducedNameOf(Name);

    if (0) {
    }
#ifdef USE_FRCVME
    else if ((Name.size() == 0) || (ReducedName == "Force_FGA5x00")) {
        Controller = new TVmeController_Force_FGA5x00();
    }
#endif
#ifdef USE_VMEDRV
    else if ((Name.size() == 0) || (ReducedName == "Bit3_617")) {
        Controller = new TVmeController_Bit3_617();
    }
    else if ((Name.size() == 0) || (ReducedName == "SBS_617")) {
        Controller = new TVmeController_Bit3_617();
    }
#endif
    else {
        throw THardwareException(
            "TDeviceFactory::CreateVmeController()",
            "No such vme controller: " + Name
        );
    }

    return Controller;
}

TCamacController* TDeviceFactory::CreateCamacController(const string& Name) throw(THardwareException)
{
    TCamacController* Controller = 0;
    string ReducedName = ReducedNameOf(Name);

    if (0) {
    }
#ifdef USE_KEK_CAMLIB
    else if ((Name.size() == 0) || (ReducedName == "Kek_Camlib")) {
        Controller = new TCamacController_Kek_Camlib();
    }
#endif
#ifdef USE_CAMDRV
    else if ((Name.size() == 0) || (ReducedName == "Toyo_CC7x00")) {
        Controller = new TCamacController_Toyo_CC7x00();
    }
#endif
    else {
        throw THardwareException(
            "TDeviceFactory::CreateCamacController()",
            "No such camac controller: " + Name
        );
    }

    return Controller;
}

