/* RoomSoftwareDevice.hh */
/* Edited by Sanshiro E. on 23 April 1999. */
/* Last updated by Sanshiro E. on 28 September 1999. */


#ifndef __RoomSoftwareDevice_hh__
#define __RoomSoftwareDevice_hh__

#include <iostream.h>
#include "RoomDefs.hh"
#include "RoomModule.hh"


class TSoftwareModule: public TModule 
{
  public:
    TSoftwareModule(void) {}
    virtual ~TSoftwareModule() {}    
};


class TPoissonGenerator: public TSoftwareModule {
  public:
    TPoissonGenerator(int NumberOfChannels = 16, int NumberOfSamples = 1, long FullScale = 4096);
    virtual ~TPoissonGenerator();
    virtual int Clear(int Channel = -1) throw(THardwareException);
    virtual int Read(int Channel, int &Data) throw(THardwareException);
    virtual int NumberOfChannels(void) throw(THardwareException);
    virtual int NumberOfSamples(void) throw(THardwareException);
  private:
    int _NumberOfChannels;
    int _NumberOfSamples;
    long _FullScale;
    int *_ReadoutCount;
};


class TSoftwareCounter: public TSoftwareModule {
  public:
    TSoftwareCounter(int NumberOfChannels = 16, int NumberOfSamples = 1, long FullScale = 4096);
    virtual ~TSoftwareCounter();
    virtual int Initialize(int InitialState = 0) throw(THardwareException);
    virtual int Clear(int Channel = -1) throw(THardwareException);
    virtual int Read(int Channel, int &Data) throw(THardwareException);
    virtual int NumberOfChannels(void) throw(THardwareException);
    virtual int NumberOfSamples(void) throw(THardwareException);
  private:
    int _NumberOfChannels;
    int _NumberOfSamples;
    long _FullScale;
    int *_ReadoutCount;
};


class TTimerInterrupter: public TSoftwareModule {
  public:
    TTimerInterrupter(long Interval_sec = 1, long Interval_usec = 0);
    virtual ~TTimerInterrupter();
    virtual int WaitData(unsigned TimeLimit = 1 /* sec */) throw(THardwareException);
    virtual int WriteRegister(int Address, int Data) throw(THardwareException);
    virtual int ReadRegister(int Address, int& Data) throw(THardwareException);
  protected:
    long _Interval_sec;
    long _Interval_usec;
};

#endif

